LocalOutlierFactor — hubness 0.21a1 documentation

# LocalOutlierFactor

class `hubness.neighbors.` `LocalOutlierFactor`(n_neighbors=20, algorithm: str = 'auto', algorithm_params: Optional[dict] = None, hubness: Optional[str] = None, hubness_params: Optional[dict] = None, leaf_size=30, metric='minkowski', p=2, metric_params=None, contamination='auto', novelty=False, n_jobs=None)

  Bases: `hubness.neighbors.base.NeighborsBase`, `hubness.neighbors.base.KNeighborsMixin`, `sklearn.neighbors.base.UnsupervisedMixin`, `sklearn.base.OutlierMixin`

  Unsupervised Outlier Detection using Local Outlier Factor (LOF)

  The anomaly score of each sample is called Local Outlier Factor. It measures the local deviation of density of a given sample with respect to its neighbors. It is local in that the anomaly score depends on how isolated the object is with respect to the surrounding neighborhood. More precisely, locality is given by k-nearest neighbors, whose distance is used to estimate the local density. By comparing the local density of a sample to the local densities of its neighbors, one can identify samples that have a substantially lower density than their neighbors. These are considered outliers.

  Parameters

    - n_neighbors (int, optional (default=20)) – Number of neighbors to use by default for `kneighbors()` queries. If n_neighbors is larger than the number of samples provided, all samples will be used.

    - algorithm ({'auto', 'hnsw', 'lsh', 'ball_tree', 'kd_tree', 'brute'}, optional) –

      Algorithm used to compute the nearest neighbors:

      - ’hnsw’ will use `HNSW`

      - ’lsh’ will use `LSH`

      - ’ball_tree’ will use `BallTree`

      - ’kd_tree’ will use `KDTree`

      - ’brute’ will use a brute-force search.

      - ’auto’ will attempt to decide the most appropriate algorithm based on the values passed to `fit()` method.

      Note: fitting on sparse input will override the setting of this parameter, using brute force.

    - algorithm_params (dict, optional) – Override default parameters of the NN algorithm. For example, with algorithm=’lsh’ and algorithm_params={n_candidates: 100} one hundred approximate neighbors are retrieved with LSH. If parameter hubness is set, the candidate neighbors are further reordered with hubness reduction. Finally, n_neighbors objects are used from the (optionally reordered) candidates.

    - TODO add all supported hubness reduction methods (#) –

    - hubness ({'mutual_proximity', 'local_scaling', 'dis_sim_local', None}, optional) – Hubness reduction algorithm - ‘mutual_proximity’ or ‘mp’ will use `MutualProximity' - 'local_scaling' or 'ls' will use :class:`LocalScaling` - ‘dis_sim_local’ or ‘dsl’ will use `DisSimLocal` If None, no hubness reduction will be performed (=vanilla kNN).

    - hubness_params (dict, optional) – Override default parameters of the selected hubness reduction algorithm. For example, with hubness=’mp’ and hubness_params={‘method’: ‘normal’} a mutual proximity variant is used, which models distance distributions with independent Gaussians.

    - leaf_size (int, optional (default=30)) – Leaf size passed to `BallTree` or `KDTree`. This can affect the speed of the construction and query, as well as the memory required to store the tree. The optimal value depends on the nature of the problem.

    - metric (string or callable, default 'minkowski') –

      metric used for the distance computation. Any metric from scikit-learn or scipy.spatial.distance can be used.

      If ‘precomputed’, the training input X is expected to be a distance matrix.

      If metric is a callable function, it is called on each pair of instances (rows) and the resulting value recorded. The callable should take two arrays as input and return one value indicating the distance between them. This works for Scipy’s metrics, but is less efficient than passing the metric name as a string.

      Valid values for metric are:

      - from scikit-learn: [‘cityblock’, ‘cosine’, ‘euclidean’, ‘l1’, ‘l2’, ‘manhattan’]

      - from scipy.spatial.distance: [‘braycurtis’, ‘canberra’, ‘chebyshev’, ‘correlation’, ‘dice’, ‘hamming’, ‘jaccard’, ‘kulsinski’, ‘mahalanobis’, ‘minkowski’, ‘rogerstanimoto’, ‘russellrao’, ‘seuclidean’, ‘sokalmichener’, ‘sokalsneath’, ‘sqeuclidean’, ‘yule’]

      See the documentation for scipy.spatial.distance for details on these metrics: https://docs.scipy.org/doc/scipy/reference/spatial.distance.html

    - p (integer, optional (default=2)) – Parameter for the Minkowski metric from `sklearn.metrics.pairwise.pairwise_distances()`. When p = 1, this is equivalent to using manhattan_distance (l1), and euclidean_distance (l2) for p = 2. For arbitrary p, minkowski_distance (l_p) is used.

    - metric_params (dict, optional (default=None)) – Additional keyword arguments for the metric function.

    - contamination ('auto' or float, optional (default='auto')) –

      The amount of contamination of the data set, i.e. the proportion of outliers in the data set. When fitting this is used to define the threshold on the scores of the samples.

      - if ‘auto’, the threshold is determined as in the original paper,

      - if a float, the contamination should be in the range [0, 0.5].

      Changed in version 0.22: The default value of `contamination` changed from 0.1 to `'auto'`.

    - novelty (boolean, default False) – By default, LocalOutlierFactor is only meant to be used for outlier detection (novelty=False). Set novelty to True if you want to use LocalOutlierFactor for novelty detection. In this case be aware that that you should only use predict, decision_function and score_samples on new unseen data and not on the training set.

    - n_jobs (int or None, optional (default=None)) – The number of parallel jobs to run for neighbors search. `None` means 1 unless in a `joblib.parallel_backend` context. `-1` means using all processors. See Glossary for more details. Affects only `kneighbors()` and `kneighbors_graph()` methods.

  `negative_outlier_factor_`

    The opposite LOF of the training samples. The higher, the more normal. Inliers tend to have a LOF score close to 1 (`negative_outlier_factor_` close to -1), while outliers tend to have a larger LOF score.

    The local outlier factor (LOF) of a sample captures its supposed ‘degree of abnormality’. It is the average of the ratio of the local reachability density of a sample and those of its k-nearest neighbors.

    Type

      numpy array, shape (n_samples,)

  `n_neighbors_`

    The actual number of neighbors used for `kneighbors()` queries.

    Type

      integer

  `offset_`

    Offset used to obtain binary labels from the raw scores. Observations having a negative_outlier_factor smaller than offset_ are detected as abnormal. The offset is set to -1.5 (inliers score around -1), except when a contamination parameter different than “auto” is provided. In that case, the offset is defined in such a way we obtain the expected number of outliers in training.

    Type

      float

  References

  1

    Breunig, M. M., Kriegel, H. P., Ng, R. T., & Sander, J. (2000, May). LOF: identifying density-based local outliers. In ACM sigmod record.

  Attributes Summary

  `decision_function` | Shifted opposite of the Local Outlier Factor of X.
  `fit_predict` | “Fits the model to the training set X and returns the labels.
  `predict` | Predict the labels (1 inlier, -1 outlier) of X according to LOF.
  `score_samples` | Opposite of the Local Outlier Factor of X.

  Methods Summary

  `fit`(X[, y]) | Fit the model using X as training data.

  Attributes Documentation

  `decision_function`

    Shifted opposite of the Local Outlier Factor of X.

    Bigger is better, i.e. large values correspond to inliers.

    The shift offset allows a zero threshold for being an outlier. Only available for novelty detection (when novelty is set to True). The argument X is supposed to contain new data: if X contains a point from training, it considers the later in its own neighborhood. Also, the samples in X are not considered in the neighborhood of any point.

    Parameters

      X (array-like, shape (n_samples, n_features)) – The query sample or samples to compute the Local Outlier Factor w.r.t. the training samples.

    Returns

      shifted_opposite_lof_scores – The shifted opposite of the Local Outlier Factor of each input samples. The lower, the more abnormal. Negative scores represent outliers, positive scores represent inliers.

    Return type

      array, shape (n_samples,)

  `fit_predict`

    “Fits the model to the training set X and returns the labels.

    Label is 1 for an inlier and -1 for an outlier according to the LOF score and the contamination parameter.

    Parameters

      - X (array-like, shape (n_samples, n_features), default=None) – The query sample or samples to compute the Local Outlier Factor w.r.t. to the training samples.

      - y (Ignored) – not used, present for API consistency by convention.

    Returns

      is_inlier – Returns -1 for anomalies/outliers and 1 for inliers.

    Return type

      array, shape (n_samples,)

  `predict`

    Predict the labels (1 inlier, -1 outlier) of X according to LOF.

    This method allows to generalize prediction to new observations (not in the training set). Only available for novelty detection (when novelty is set to True).

    Parameters

      X (array-like, shape (n_samples, n_features)) – The query sample or samples to compute the Local Outlier Factor w.r.t. to the training samples.

    Returns

      is_inlier – Returns -1 for anomalies/outliers and +1 for inliers.

    Return type

      array, shape (n_samples,)

  `score_samples`

    Opposite of the Local Outlier Factor of X.

    It is the opposite as bigger is better, i.e. large values correspond to inliers.

    Only available for novelty detection (when novelty is set to True). The argument X is supposed to contain new data: if X contains a point from training, it considers the later in its own neighborhood. Also, the samples in X are not considered in the neighborhood of any point. The score_samples on training data is available by considering the the `negative_outlier_factor_` attribute.

    Parameters

      X (array-like, shape (n_samples, n_features)) – The query sample or samples to compute the Local Outlier Factor w.r.t. the training samples.

    Returns

      opposite_lof_scores – The opposite of the Local Outlier Factor of each input samples. The lower, the more abnormal.

    Return type

      array, shape (n_samples,)

  Methods Documentation

  `fit`(X, y=None) → hubness.neighbors.lof.LocalOutlierFactor

    Fit the model using X as training data.

    Parameters

      - X ({array-like, sparse matrix, BallTree, KDTree}) – Training data. If array or matrix, shape [n_samples, n_features], or [n_samples, n_samples] if metric=’precomputed’.

      - y (Ignored) – not used, present for API consistency by convention.

    Returns

      self

    Return type

      object